/* 
 * PiTiVi
 * Copyright (C) <2004> Guillaume Casanova <casano_g@epita.fr>
 *			Stephan Bloch <bloch_s@epitech.net>                     
 *
 * This software has been written in EPITECH <http://www.epitech.net>
 * EPITECH is a computer science school in Paris - FRANCE -
 * under the direction of Flavien Astraud and Jerome Landrieu.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "pitivi.h"
#include "pitivi-debug.h"
#include "pitivi-effectswindow.h"
#include "pitivi-dragdrop.h"
#include "pitivi-effects_cat.h"
#include "pitivi-sourcefile.h"

static	GtkWindowClass *parent_class = NULL;

gchar* labels[PITIVI_EFFECT_NBCAT_TYPE + 1] = {
  PITIVI_TRANSITION_EFFECT_LABEL,
  PITIVI_VIDEO_EFFECT_LABEL,
  PITIVI_AUDIO_EFFECT_LABEL,
  0
};

struct _PitiviEffectsWindowPrivate
{
  /* instance private members */
  gboolean		dispose_has_run;
  guint			notebook_id;
  GtkWidget		*notebook;
  PitiviEffectsTree	*trees[PITIVI_EFFECT_NBCAT_TYPE];
  PitiviSourceFile	*dndse;
  GtkWidget		*timelinewin;
};

/*
 * forward definitions
 */

enum {
    PITIVI_ICON_COLUMN,
    PITIVI_TEXT_COLUMN,
    PITIVI_POINTER_COLUMN,
    PITIVI_NB_COLUMN
};

static GtkTargetEntry TargetEntries[] =
{
  { "pitivi/timeline/sourceeffect", GTK_TARGET_SAME_APP, DND_TARGET_EFFECTSWIN },
  { "pitivi/media/sourceeffect", GTK_TARGET_SAME_APP, DND_TARGET_EFFECTSWIN }
};

static gint iNbTargetEntries = G_N_ELEMENTS (TargetEntries);

/*
 **********************************************************
 * Signals  					          *
 *							  *
 **********************************************************
*/

enum
  {
    SELECT_MEDIA_SOURCE_SIGNAL,
    LAST_SIGNAL
  };

/* static guint	      effects_signals[LAST_SIGNAL] = {0}; */

/*
 * Insert "added-value" functions here
 */

PitiviEffectsWindow *
pitivi_effectswindow_new (PitiviMainApp *mainapp)
{
  PitiviEffectsWindow	*effectswindow;

  effectswindow = (PitiviEffectsWindow *) g_object_new(PITIVI_EFFECTSWINDOW_TYPE, 
						       "mainapp", mainapp, NULL);
  g_assert(effectswindow != NULL);  
  return effectswindow;
}

static void
pitivi_effectswindow_insert_newtab (GtkNotebook *notebook, PitiviEffectsTree *tree)
{
  GtkWidget *widget;
  
  gtk_container_add (GTK_CONTAINER (notebook), tree->scroll);
  gtk_container_add (GTK_CONTAINER (tree->scroll), tree->treeview);
  widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), tree->order);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK ( notebook ), widget, GTK_WIDGET (tree->label));
  gtk_label_set_justify (GTK_LABEL ( tree->label ), GTK_JUSTIFY_LEFT);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (tree->scroll),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
}

static GObject *
pitivi_effectswindow_constructor (GType type,
			     guint n_construct_properties,
			     GObjectConstructParam * construct_properties)
{
  GObject *obj;
  int  count;
  GtkWidget *main_vbox;

    /* Invoke parent constructor. */
  PitiviEffectsWindowClass *klass;
  PitiviEffectsWindow	   *self;

  GObjectClass *parent_class;
  klass = PITIVI_EFFECTSWINDOW_CLASS (g_type_class_peek (PITIVI_EFFECTSWINDOW_TYPE));
  parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
  obj = parent_class->constructor (type, n_construct_properties,
				   construct_properties);
  self = (PitiviEffectsWindow *) obj;

  /* do stuff. */
  
  main_vbox = gtk_vbox_new (FALSE, 0);
  self->private->notebook = gtk_notebook_new ();
  gtk_widget_set_usize (main_vbox, PITIVI_EFFECTS_WIN_SIZEX, PITIVI_EFFECTS_WIN_SIZEY);
  gtk_box_pack_start (GTK_BOX (main_vbox), self->private->notebook, TRUE, TRUE, 0);
  gtk_container_add (GTK_CONTAINER (self), main_vbox);
  
  for (count = 0; count < PITIVI_EFFECT_NBCAT_TYPE - 1; count++)
    {
      self->private->trees[count] = g_new0 (PitiviEffectsTree, 1);
      self->private->trees[count]->window = GTK_WIDGET (self);
      self->private->trees[count]->label = gtk_label_new (labels[count]);
      self->private->trees[count]->treeview = gtk_tree_view_new ();
      self->private->trees[count]->scroll = gtk_scrolled_window_new (NULL, NULL);
      self->private->trees[count]->order = count;
      pitivi_effectstree_set_gst (self,
				  self->private->trees[count], 
				  count+1, 
				  ((PitiviWindows *)self)->mainapp->global_settings);
      pitivi_effectswindow_insert_newtab (GTK_NOTEBOOK (self->private->notebook), self->private->trees[count]);
      gtk_tree_view_expand_all (GTK_TREE_VIEW (self->private->trees[count]->treeview));
    }
  self->private->timelinewin = (GtkWidget *) pitivi_mainapp_get_timelinewin (((PitiviWindows *)self)->mainapp);
  return obj;
}


static void
pitivi_effectswindow_instance_init (GTypeInstance * instance, gpointer g_class)
{
  PitiviEffectsWindow *self = (PitiviEffectsWindow *) instance;
  self->private = g_new0(PitiviEffectsWindowPrivate, 1);
  self->private->dispose_has_run = FALSE;
  gtk_window_set_title (GTK_WINDOW (self), PITIVI_EFFECTS_DF_TITLE);
  gtk_window_set_default_size (GTK_WINDOW (self), PITIVI_EFFECTS_DF_WIN_WIDTH, PITIVI_EFFECTS_DF_WIN_HEIGHT); 
}

void
pitivi_effects_action_on_colexp (GtkTreeView *treeview, 
				 GtkTreeIter *TreeIter, 
				 gchar *icon, 
				 gpointer data)
{
  GdkPixbuf           *pixbuf;
  GtkTreeModel	      *model;
  PitiviEffectsTree   *effectstree;
  gchar		      *name;
   
  effectstree = (PitiviEffectsTree *) data;
  model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
  pixbuf = gtk_widget_render_icon(effectstree->window, icon, GTK_ICON_SIZE_MENU, NULL);  
  gtk_tree_model_get (GTK_TREE_MODEL (model), TreeIter, PITIVI_TEXT_COLUMN, &name, -1);
  gtk_tree_store_set(GTK_TREE_STORE (model), TreeIter,
		     PITIVI_ICON_COLUMN, pixbuf,
		     -1);
}

void
effectstree_on_row_activated (GtkTreeView        *treeview,
			      GtkTreePath        *path,
			      GtkTreeViewColumn  *col,
			      gpointer            data)
{
  GtkTreeSelection    *selection;
  PitiviEffectsWindow *self;
  GtkTreeModel	      *model;
  PitiviSourceFile    *info;
  GtkTreeIter	      iter;
  
  self = (PitiviEffectsWindow *) gtk_widget_get_toplevel (GTK_WIDGET (treeview));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
  if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
    PITIVI_WARNING("No elements selected!");
    return;
  }
  
  gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, PITIVI_POINTER_COLUMN, &info, -1);
  if ( data && info )
    {
      g_signal_emit_by_name (GTK_OBJECT (pitivi_mainapp_get_timelinewin(((PitiviWindows *)self)->mainapp)), 
			     "associate-effect-to-media", info);
    }
}

void
effectstree_on_row_collapsed (GtkTreeView *treeview, 
			      GtkTreeIter *TreeIter, 
			      GtkTreePath *arg2, 
			      gpointer data)
{
  pitivi_effects_action_on_colexp (treeview, TreeIter, PITIVI_STOCK_EFFECT_CAT, data);
}


void
effectstree_on_row_expanded (GtkTreeView *treeview,
			     GtkTreeIter *TreeIter,
			     GtkTreePath *arg2, 
			     gpointer data)
{
  pitivi_effects_action_on_colexp (treeview, TreeIter, PITIVI_STOCK_EFFECT_CAT_OPEN, data);
}

/* Insertion on Node */

void
pitivi_effectstree_insert_node (PitiviEffectsTree *tree_effect,
				 GtkTreeIter *child,
				 GtkTreeIter *parent,
				 const gchar *name,
				 gchar *icon,
				 gpointer data)
{
  GdkPixbuf *pixbuf;

  pixbuf = gtk_widget_render_icon(tree_effect->window, icon, GTK_ICON_SIZE_MENU, NULL);
  gtk_tree_store_append (tree_effect->model, child, parent);
  gtk_tree_store_set(tree_effect->model, child,
		     PITIVI_ICON_COLUMN, pixbuf,
		     PITIVI_TEXT_COLUMN, name,
		     PITIVI_POINTER_COLUMN, data,
		     -1);
  tree_effect->pixbuf = pixbuf;
}

void
pitivi_effectstree_insert_transition (PitiviEffectsWindow *win,
				      PitiviEffectsTree *tree_effect,
				      GtkTreeIter *child,
				      GtkTreeIter *parent,
				      const gchar *name,
				      const gchar *desc,
				      gchar *icon,
				      gint transitionid,
				      gpointer data)
{
  GdkPixbuf *pixbuf;
  PitiviSourceFile *se;
  
  pixbuf = gtk_widget_render_icon  (tree_effect->window, icon, GTK_ICON_SIZE_LARGE_TOOLBAR, NULL);
  se = pitivi_sourcefile_new_transition ((gchar *) name, GST_ELEMENT_FACTORY (data), 
					 pixbuf, (gchar *) desc, transitionid,
					 PITIVI_WINDOWS(win)->mainapp);
  gtk_tree_store_append (tree_effect->model, child, parent);
  gtk_tree_store_set(tree_effect->model, child,
		     PITIVI_ICON_COLUMN, pixbuf,
		     PITIVI_TEXT_COLUMN, name,
		     PITIVI_POINTER_COLUMN, se,
		     -1);
}


void
pitivi_effectstree_insert_effect (PitiviEffectsWindow *win,
				  PitiviEffectsTree *tree_effect,
				  GtkTreeIter *child,
				  GtkTreeIter *parent,
				  const gchar *name,
				  const gchar *desc,
				  gchar *icon,
				  gpointer data)
{
  GdkPixbuf *pixbuf;
  PitiviSourceFile *se;
  
  pixbuf = gtk_widget_render_icon  (tree_effect->window, icon, GTK_ICON_SIZE_LARGE_TOOLBAR, NULL);
  se = pitivi_sourcefile_new_effect ((gchar *) name, GST_ELEMENT_FACTORY (data), 
				     pixbuf, (gchar *) desc, PITIVI_WINDOWS(win)->mainapp);
  gtk_tree_store_append (tree_effect->model, child, parent);
  gtk_tree_store_set(tree_effect->model, child,
		     PITIVI_ICON_COLUMN, pixbuf,
		     PITIVI_TEXT_COLUMN, name,
		     PITIVI_POINTER_COLUMN, se,
		     -1);
}

/**************************************************************
 * Callbacks Signal Drag and Drop          		      *
 * This callbacks are used to motion get or delete  data from *
 * drag							      *
 **************************************************************/

void
slide_effects_info (PitiviEffectsWindow *self, gint64 length, gchar *path)
{
  struct _Pslide
  {
    gint64 length;
    gchar  *path;
  } slide;
  
  slide.length = length;
  slide.path = path;
  g_signal_emit_by_name (self->private->timelinewin, "drag-source-begin", &slide);
}


static void
pitivi_effectswindow_drag_data_get (GtkWidget          *widget,
				    GdkDragContext     *context,
				    GtkSelectionData   *selection_data,
				    guint               info,
				    guint32             time,
				    gpointer user_data)
{
  PitiviEffectsWindow *self = (PitiviEffectsWindow *) gtk_widget_get_toplevel (widget);
  if (self && self->private->dndse)
    gtk_selection_data_set (selection_data, 
			    selection_data->target, 
			    8, 
			    (void *) &self->private->dndse,
			    sizeof (PitiviSourceFile *));
}

static void
pitivi_effectswindow_drag_begin (GtkWidget		*widget,
				 GdkDragContext		*context,
				 gpointer		user_data)
{
  PitiviEffectsWindow *self = (PitiviEffectsWindow *) gtk_widget_get_toplevel (widget);
  PitiviSourceFile    *se;
  GtkTreeSelection    *selection;
  GtkTreeModel	      *model;
  GtkTreeIter	      iter;
  gint64	      size;

  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
  if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
    PITIVI_WARNING("No elements selected!");
    return;
  }
  gtk_tree_model_get (model, &iter, PITIVI_POINTER_COLUMN, &se, -1);
  self->private->dndse = se;
  size = ((gint64)50000000000LL);
  slide_effects_info ( self, size, "transition" );
}

/**************************************************************
 * Effects Insertion					      *
 *							      *
 *							      *
 **************************************************************/

gchar	*get_icon_fx(G_CONST_RETURN gchar *name, gint type)
{
  gchar	*icon_fx = NULL;
  gint	id_tab;

  switch(type)
    {
    case 1:
      {
	id_tab = 0;
	while (video_effect_tab[id_tab].name)
	  {
	    if (!strcmp(video_effect_tab[id_tab].name, name))
	      {
		icon_fx = video_effect_tab[id_tab].image;
		return (icon_fx);
	      }
	    else
	      {
		icon_fx = PITIVI_STOCK_EFFECT_TV;
	      }
	    id_tab++;
	  }
      }
      break;
    case 2:
      {
	id_tab = 0;
	while (audio_effect_tab[id_tab].name)
	  {

	    if (!strcmp(audio_effect_tab[id_tab].name, name))
	      {
		icon_fx = audio_effect_tab[id_tab].image;
		return (icon_fx);
	      }
	    else
	      {
		icon_fx = PITIVI_STOCK_EFFECT_SOUND;
	      }
	    id_tab++;
	  }
      }
      break;
    }
  return (icon_fx);
}

void
insert_video_effects_on_tree (PitiviEffectsWindow *win,
			      PitiviEffectsTree *tree_effect, 
			      GtkTreeIter *child, 
			      GList *settingslist)
{
  const gchar		*klass;
  const gchar		*effectname;
  const gchar		*desc;
  GtkTreeIter		video_iter[2];
  GList			*fx_video = NULL;
  G_CONST_RETURN gchar	*name;
  gchar			*icon_fx;

  pitivi_effectstree_insert_node (tree_effect, 
				  &tree_effect->treeiter,  
				  NULL,  
				  "Simple Effects",  
				  PITIVI_STOCK_EFFECT_CAT, 
				  NULL);
  
  /* On recupere la liste des effets video via la structure self */
  while (settingslist)
    {
      fx_video = g_list_append (fx_video, settingslist->data);
      settingslist = settingslist->next;
    }
  
  /* On creer deux sous categories */
  pitivi_effectstree_insert_node (tree_effect, &video_iter[0], NULL,
				   "Tv Effects", PITIVI_STOCK_EFFECT_CAT, NULL);
  pitivi_effectstree_insert_node (tree_effect, &video_iter[1], NULL,
				   "Video Effects", PITIVI_STOCK_EFFECT_CAT, NULL);
  /* On insere les elements video dans le tree pour le menu des effets */
  while (fx_video)
    {
      name = gst_plugin_feature_get_name(GST_PLUGIN_FEATURE(fx_video->data));
      klass = gst_element_factory_get_klass (fx_video->data);
      effectname = gst_element_factory_get_longname (fx_video->data);
      desc = gst_element_factory_get_description (fx_video->data);
      if (!strncmp (klass, "Filter/Effect/Video", 19))
	{
	  gchar *idx;

	  if ((idx = strstr (effectname, "TV")))
	    {
	      *idx = '\0';
	      icon_fx = get_icon_fx(name, 1);
	      pitivi_effectstree_insert_effect (win,
					       tree_effect, 
					       child,
					       &video_iter[0],
					       effectname,
					       "video/effect", 
					       icon_fx, 
					       fx_video->data);
	    }
	  else if ((idx = strstr (effectname, "ideo")))
	    {
	      icon_fx = get_icon_fx(name, 1);
	      pitivi_effectstree_insert_effect (win,
					       tree_effect, 
					       child,
					       &video_iter[1],
					       effectname,
					       "video/effect", 
					       icon_fx, 
					       fx_video->data);
	    }
	  else
	    {
	      icon_fx = get_icon_fx(name, 1);
	      pitivi_effectstree_insert_effect (win,
					       tree_effect, 
					       child,
					       &tree_effect->treeiter,
					       effectname,
					       "video/effect", 
					       icon_fx, 
					       fx_video->data);
	    } 
	}
      fx_video = fx_video->next;
    }
}

void
insert_audio_effects_on_tree (PitiviEffectsWindow *win,
			      PitiviEffectsTree *tree_effect, 
			      GtkTreeIter *child, 
			      GList *settingslist)
{
  const gchar		*klass;
  const gchar		*effectname;
  const gchar		*desc;
  GList			*fx_audio = NULL;
  G_CONST_RETURN gchar	*name;
  gchar			*icon_fx;

  pitivi_effectstree_insert_node (tree_effect, 
				  &tree_effect->treeiter,  
				  NULL,  
				  "Simple Effects",  
				  PITIVI_STOCK_EFFECT_CAT, NULL);
  while ( settingslist )
    {
      fx_audio = g_list_append (fx_audio,  settingslist->data);
      settingslist = settingslist->next;
    }
  while ( fx_audio )
    {
      name = gst_plugin_feature_get_name(GST_PLUGIN_FEATURE(fx_audio->data));
      klass = gst_element_factory_get_klass (fx_audio->data);
      effectname = gst_element_factory_get_longname (fx_audio->data);
      desc = gst_element_factory_get_description (fx_audio->data);
      if (!strncmp (klass, "Filter/Effect/Audio", 19))
	{
	  icon_fx = get_icon_fx(name, 2);
	  pitivi_effectstree_insert_effect (win,
					   tree_effect, 
					   child, 
					   &tree_effect->treeiter,
					   effectname,
					   "audio/effect", 
					   icon_fx, 
					   fx_audio->data);
	}
      fx_audio = fx_audio->next;
    }
}

void
insert_transition_effects_on_tree (PitiviEffectsWindow *win,
				   PitiviEffectsTree *tree_effect, 
				   GtkTreeIter *child, 
				   GList *settingslist)
{
  const gchar			*klass;
  const gchar			*effectname;
  const gchar			*desc;
  gint				nb, nb_tcat;
  GList				*fx_transition = NULL;
  GtkTreeIter			Trans_iter[18];

  /* On recupere la liste des effets de transition via la structure self */

  while (settingslist)
    {
      fx_transition = g_list_append(fx_transition, settingslist->data);
      settingslist = settingslist->next;
    }

  while ( fx_transition )
    {
      klass = gst_element_factory_get_klass ( fx_transition->data );
      effectname = gst_element_factory_get_longname (  fx_transition->data );
      desc = gst_element_factory_get_description (  fx_transition->data );
      if (strstr (effectname, "SMPTE"))
	break;
      fx_transition = fx_transition->next;
    }

  /* On creer 18 sous categories */
  
  for (nb_tcat = 0; nb_tcat < PITIVI_LAST_WIPE; nb_tcat++)
    {
      pitivi_effectstree_insert_node (tree_effect, &Trans_iter[nb_tcat], NULL,
				       transition_cat[nb_tcat], PITIVI_STOCK_EFFECT_CAT, NULL);
      for (nb = 0; nb < (sizeof (tab_category) / sizeof (PitiviTransProp)); nb++)
	{
	  /* On test les elements du tableau et on les insere dans les differentes categories */
	  
	  if (nb_tcat == tab_category[nb].id_categorie && tab_category[nb].name)
	    {
	      pitivi_effectstree_insert_transition (win,
						    tree_effect, 
						    child, 
						    &Trans_iter[nb_tcat],
						    tab_category[nb].name,
						    "transition",
						    tab_category[nb].image,
						    tab_category[nb].smpte_num,
						    fx_transition->data);
	    }
	}
    }
  pitivi_effectstree_insert_node (tree_effect, 
				  &tree_effect->treeiter,  
				  NULL,  
				  "Simple Effects",  
				  PITIVI_STOCK_EFFECT_CAT, 
				  "");
}

void
pitivi_effectstree_set_gst (PitiviEffectsWindow *win,
			    PitiviEffectsTree *tree_effect,
			    PitiviEffectsTypeEnum eneffects,  
			    PitiviSettings *setting)
{
  GtkWidget			*self;
  GtkCellRenderer		*pCellRenderer;
  GtkTreeViewColumn		*pColumn;
  GdkPixbuf			*pixbuf;
  
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tree_effect->treeview), FALSE);
  tree_effect->model = gtk_tree_store_new ( PITIVI_NB_COLUMN,
					    GDK_TYPE_PIXBUF,
					    G_TYPE_STRING,
					    G_TYPE_POINTER,
					    -1);
  
  gtk_tree_view_set_model(GTK_TREE_VIEW(tree_effect->treeview), GTK_TREE_MODEL(tree_effect->model));
  
  /* On check le type d'effet a inserer (video/audio/transition) */

  switch (eneffects)
    {
      GtkTreeIter child;

    case PITIVI_EFFECT_VIDEO_TYPE:
      insert_video_effects_on_tree (win, tree_effect, &child, setting->video_effects);
      break;
    case PITIVI_EFFECT_AUDIO_TYPE:
      insert_audio_effects_on_tree (win, tree_effect, &child, setting->audio_effects);
      break;
    case PITIVI_EFFECT_TRANSITION_TYPE:
      insert_transition_effects_on_tree (win, tree_effect, &child, setting->transition_effects);
      break;
    case PITIVI_EFFECT_NBCAT_TYPE:
      break;
    }
  
  pCellRenderer = gtk_cell_renderer_pixbuf_new();
  pColumn = gtk_tree_view_column_new_with_attributes("",
						     pCellRenderer,
						     "pixbuf",
						     PITIVI_ICON_COLUMN,
						     NULL);
  
  gtk_tree_view_append_column(GTK_TREE_VIEW(tree_effect->treeview), pColumn);
  
  pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes("",
						     pCellRenderer,
						     "text",
						     PITIVI_TEXT_COLUMN,
						     NULL);
  
  g_signal_connect (tree_effect->treeview, "row-expanded", G_CALLBACK ( effectstree_on_row_expanded ),\
		    (gpointer) tree_effect);
  g_signal_connect (tree_effect->treeview, "row-collapsed", G_CALLBACK ( effectstree_on_row_collapsed ),\
		    (gpointer) tree_effect);
    
  gtk_tree_view_append_column(GTK_TREE_VIEW (tree_effect->treeview), pColumn);
  
  self = gtk_widget_get_toplevel (tree_effect->treeview);
  
  g_signal_connect (tree_effect->treeview, "drag_data_get",	      
		    G_CALLBACK (pitivi_effectswindow_drag_data_get), self);
  g_signal_connect (tree_effect->treeview, "drag_begin",	      
		    G_CALLBACK (pitivi_effectswindow_drag_begin), self);
    g_signal_connect (tree_effect->treeview, "row-activated", 
		      G_CALLBACK  ( effectstree_on_row_activated ), 
		      self);
  if (eneffects == PITIVI_EFFECT_TRANSITION_TYPE)
    {
      // Drag 'n Drop Activation
      
      gtk_drag_source_set (GTK_WIDGET (tree_effect->treeview), 
			   GDK_BUTTON1_MASK,
			   TargetEntries, iNbTargetEntries, 
			   GDK_ACTION_COPY);
      pixbuf = gtk_widget_render_icon (GTK_WIDGET (tree_effect->treeview), PITIVI_STOCK_HAND, 
				       GTK_ICON_SIZE_DND, NULL);
      gtk_drag_source_set_icon_pixbuf (GTK_WIDGET (tree_effect->treeview), pixbuf);
    }
  else
    {
      gtk_tree_view_enable_model_drag_source (GTK_TREE_VIEW (tree_effect->treeview),
					      GDK_BUTTON1_MASK,
					      TargetEntries, 
					      iNbTargetEntries,
					      GDK_ACTION_COPY|GDK_ACTION_MOVE);
    }
}


/**************************************************************
 * Window Effects Initialization			      *
 * and Construction					      *
 *							      *
 **************************************************************/


static void
pitivi_effectswindow_dispose (GObject *object)
{
  PitiviEffectsWindow	*self = PITIVI_EFFECTSWINDOW(object);

  /* If dispose did already run, return. */
  if (self->private->dispose_has_run)
    return;
  
  /* Make sure dispose does not run twice. */
  self->private->dispose_has_run = TRUE;	

  /* 
   * In dispose, you are supposed to free all types referenced from this 
   * object which might themselves hold a reference to self. Generally, 
   * the most simple solution is to unref all members on which you own a 
   * reference. 
   */
 
  G_OBJECT_CLASS (parent_class)->dispose (object);
}

static void
pitivi_effectswindow_finalize (GObject *object)
{
  PitiviEffectsWindow	*self = PITIVI_EFFECTSWINDOW(object);

  /* 
   * Here, complete object destruction. 
   * You might not need to do much... 
   */

  g_free (self->private);
  G_OBJECT_CLASS (parent_class)->finalize (object);
}

static gboolean
pitivi_effectswindow_delete_event ( GtkWidget  *widget,
				    GdkEventAny *event )
{
  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
  gtk_widget_hide (widget);
  pitivi_timelinewindow_windows_set_action (pitivi_mainapp_get_timelinewin (((PitiviWindows *) widget)->mainapp), 
					    "EffectWindows", FALSE);
  return TRUE;
}

static void
pitivi_effectswindow_class_init (gpointer g_class, gpointer g_class_data)
{
  GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);
  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (g_class);
  
  parent_class = g_type_class_peek_parent (g_class);
  
  gobject_class->constructor = pitivi_effectswindow_constructor;
  gobject_class->dispose = pitivi_effectswindow_dispose;
  gobject_class->finalize = pitivi_effectswindow_finalize;
  widget_class->delete_event = pitivi_effectswindow_delete_event;
}

GType
pitivi_effectswindow_get_type (void)
{
  static GType type = 0;
 
  if (type == 0)
    {
      static const GTypeInfo info = {
	sizeof (PitiviEffectsWindowClass),
	NULL,			/* base_init */
	NULL,			/* base_finalize */
	pitivi_effectswindow_class_init,	/* class_init */
	NULL,			/* class_finalize */
	NULL,			/* class_data */
	sizeof (PitiviEffectsWindow),
	0,			/* n_preallocs */
	pitivi_effectswindow_instance_init	/* instance_init */
      };
      type = g_type_register_static (PITIVI_WINDOWS_TYPE,
				     "PitiviEffectsWindowType", &info, 0);
    }
  return type;
}
